#ifndef __FAT_H
#define __FAT_H

/* FAT boot sector (from Wikipedia):
0 	3 	Jump instruction (to skip over header on boot)
3 	8 	OEM Name (padded with spaces)
11 	2 	Bytes per sector
13 	1 	Sectors per cluster
14 	2 	Reserved sector count (including boot sector)
16 	1 	Number of file allocation tables
17 	2 	Number of root directory entries
19 	2 	Total sectors (bits 0-15)
21 	1 	Media descriptor
22 	2 	Sectors per file allocation table (FAT16)
24 	2 	Sectors per track
26 	2 	Number of heads
28 	4 	Hidden sectors
32 	4 	Total sectors (bits 16-47)
36 	4 	Sectors per file allocation table (FAT32)
36 	1 	Physical drive number (FAT16)
37 	1 	Current head (FAT16)
38 	1 	Signature (FAT16)
39 	4 	ID (FAT16)
43 	11 	Volume Label
54 	8 	FAT file system type (e.g. FAT, FAT12, FAT16, FAT32)
*/

#define FAT_TYPE_UNKNOWN 0
#define FAT_TYPE_12      12
#define FAT_TYPE_16      16
#define FAT_TYPE_32      32

typedef unsigned int cluster_t;

/* root directory entry */
typedef struct _fat_dir_entry_ {
  char filename[13];
  int attribute;
  cluster_t cluster;
  int filesize;
  struct _fat_dir_entry_ *next;
} fat_dir_entry_t;


typedef struct _fat_ {
  char label[12];
  int type;
  int sectors_per_cluster;
  int sectors_per_FAT;
  int number_of_FAT;
  int root_size;   /* number of entries in root directory */
  int FAT_sector;  /* first FAT sector */
  int root_sector; /* first root directory sector */
  cluster_t root_cluster; /* first root directory cluster: FAT32 only */
  int data_sector; /* first data sector */
  int length;      /* length in sectors */
  fat_dir_entry_t *root; /* root directory */
  void *table; /* FAT or single sector cache in FAT32 */
  int fat_sector;  /* FAT sector number in cache for FAT32 */
} fat_t;

/* attibutes */
#define FAT_ATTR_READONLY  1
#define FAT_ATTR_HIDDEN    2
#define FAT_ATTR_SYSTEM    4
#define FAT_ATTR_LABEL     8
#define FAT_ATTR_SUBDIR   16
#define FAT_ATTR_ARCHIVE  32

/* initialize the fat filesystem */
int fat_init(fat_t *fat, unsigned int sector_start, int dtype);

/* open a file in the FAT */
cluster_t fat_open_file(fat_t *fat, int *file_size, char const *filename);

/* read a file in the FAT */
int fat_read_file(fat_t *fat, cluster_t cluster, void *buffer);

cluster_t fat_read_cluster(fat_t *fat, cluster_t cluster, void *buffer);
cluster_t fat_next(fat_t *fat, cluster_t cluster);
#endif
